@@ -3,9 +3,9 @@
  *
  * Copyright (C) 2019 commend.com - Christian Spielberger
  */
+#include <stdint.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -25,7 +25,10 @@
  * stream. The audio source is not faded.
  */
 
-#define DEFAULT_FADE_TIME 160  /*default fading time in ms*/
+enum {
+	DEFAULT_FADE_TIME=160,  /**< Default fade time [ms]                  */
+	PTIME=40,               /**< Packet time for ausrc reading [ms]      */
+};
 
 /**
  * State machine. See:
@@ -79,31 +82,32 @@ struct mixstatus {
 
 	char *module;                   /**< Audio source module name        */
 	char *param;                    /**< Parameter for audio source      */
-	enum mixmode mode;              /**< Current mix mode                */
-	enum mixmode nextmode;          /**< Next mix mode                   */
 	float minvol;                   /**< Minimum audio stream volume     */
 	float ausvol;                   /**< Volume for mixed audio source   */
-	uint32_t ptime;                 /**< Stream packet time              */
-	size_t sampc_strm;              /**< Stream sample count frame       */
-	size_t nbytes_strm;             /**< Stream bytes per frame          */
-	size_t sampc_max;               /**< Max sample count per frame      */
-	size_t nbytes_max;              /**< Max bytes per frame             */
+	size_t nmix;                    /**< Size of mixer buffer [bytes]    */
 	uint16_t i_fade;                /**< Fade-in/-out counter            */
 	uint16_t n_fade;                /**< Fade-in/-out steps              */
 	float delta_fade;               /**< linear delta accumulation       */
-	struct aufilt_prm prm;          /**< Audio filter parameter          */
+	int16_t *mixbuf;		/**< Mixer buffer                    */
+	uint32_t fadetime;	        /**< Fade time in milliseconds       */
 
+	enum mixmode mode;              /**< Current mix mode                */
+	enum mixmode nextmode;          /**< Next mix mode                   */
+	mtx_t *mtx_mode;		/**< Mutex for mode changes          */
+
+	/* thread ausrc */
+	struct aubuf *aubuf;	        /**< Buffer for resampled ausrc      */
 	struct auresamp resamp;         /**< Optional audio resampler        */
+	size_t nres;                    /**< Size of resample buffer [bytes] */
 	void *sampvrs;                  /**< Optional resample buffer        */
-
-	struct aubuf *aubuf;
-	bool aubuf_started;
-	int16_t *rbuf;
+	struct aufilt_prm prm;          /**< Audio filter parameter          */
+	mtx_t *mtx_ausrc;               /**< Mutex for ausrc thread          */
 };
 
 
 struct mixausrc_enc {
 	struct aufilt_enc_st af;  /* inheritance */
+	char *cname;
 
 	struct mixstatus st;
 	struct le le_priv;
@@ -112,6 +116,7 @@ struct mixausrc_enc {
 
 struct mixausrc_dec {
 	struct aufilt_dec_st af;  /* inheritance */
+	char *cname;
 
 	struct mixstatus st;
 	struct le le_priv;
@@ -122,7 +127,7 @@ static struct list encs;
 static struct list decs;
 
 
-static int stop_ausrc(struct mixstatus *st);
+static void stop_ausrc(struct mixstatus *st);
 
 
 static int init_aubuf(struct mixstatus *st)
@@ -133,35 +138,50 @@ static int init_aubuf(struct mixstatus *st)
 	size_t wishsz;
 
 	conf_get_u32(conf_cur(), "mixausrc_wish_size", &v);
-	wishsz = v * st->nbytes_strm;
+	wishsz = v * st->nres;
 	maxsz = 2 * wishsz;
 
 	st->aubuf = mem_deref(st->aubuf);
-	st->aubuf_started = false;
 
 	err = aubuf_alloc(&st->aubuf, wishsz, maxsz);
 	if (err) {
 		warning("mixausrc: Could not allocate aubuf. wishsz=%lu, "
 				"maxsz=%lu (%m)\n", wishsz, maxsz, err);
-		goto out;
+		return err;
 	}
 
 	aubuf_set_live(st->aubuf, false);
-	if (st->rbuf)
-		return 0;
 
-	st->rbuf = mem_zalloc(st->nbytes_strm, NULL);
-	if (!st->rbuf) {
-		warning("mixausrc: Could not allocate rbuf.\n");
+	struct pl *id = pl_alloc_str("mixausrc");
+	if (!id) {
+		err = ENOMEM;
 		goto out;
 	}
 
+	aubuf_set_id(st->aubuf, id);
+	mem_deref(id);
+
 out:
+	if (err)
+		st->aubuf = mem_deref(st->aubuf);
+
 	return err;
 }
 
 
-static int process_resamp(struct mixstatus *st, const struct auframe *afsrc)
+static int init_mixbuf(struct mixstatus *st)
+{
+	st->mixbuf = mem_deref(st->mixbuf);
+	st->mixbuf = mem_zalloc(st->nmix, NULL);
+	if (!st->mixbuf)
+		return ENOMEM;
+
+	return 0;
+}
+
+
+static int process_resamp(struct mixstatus *st, struct auframe *afres,
+			  const struct auframe *afsrc)
 {
 	int err = 0;
 	if (afsrc->fmt != AUFMT_S16LE) {
@@ -170,7 +190,7 @@ static int process_resamp(struct mixstatus *st, const struct auframe *afsrc)
 		return EINVAL;
 	}
 
-	if (!st->resamp.resample) {
+	if (!st->resamp.resample || !st->sampvrs) {
 		debug("mixausrc: resample ausrc %u/%u -> %u/%u\n",
 				st->ausrc_prm.srate, st->ausrc_prm.ch,
 				st->prm.srate, st->prm.ch);
@@ -184,26 +204,17 @@ static int process_resamp(struct mixstatus *st, const struct auframe *afsrc)
 		}
 
 		st->sampvrs = mem_deref(st->sampvrs);
-		st->sampvrs = mem_zalloc(st->nbytes_max, NULL);
-		if (!st->sampvrs) {
-			warning("mixausrc: could not alloc resample buffer\n");
+		st->sampvrs = mem_zalloc(st->nres, NULL);
+		if (!st->sampvrs)
 			return ENOMEM;
-		}
-	}
-
-	if (st->resamp.resample) {
-		size_t sampc = st->sampc_max;
-		err = auresamp(&st->resamp, st->sampvrs, &sampc,
-			       afsrc->sampv, afsrc->sampc);
-
-		if (sampc != st->sampc_strm) {
-			warning("mixausrc: unexpected sample count "
-					"%u vs. %u\n", sampc, st->sampc_strm);
-			st->sampc_strm = sampc;
-			st->nbytes_strm = aufmt_sample_size(afsrc->fmt)*sampc;
-		}
 	}
 
+	afres->sampv = st->sampvrs;
+	afres->sampc = st->nres / aufmt_sample_size(st->prm.fmt);
+	afres->srate = st->prm.srate;
+	afres->ch    = st->prm.ch;
+	err = auresamp(&st->resamp, afres->sampv, &afres->sampc,
+		       afsrc->sampv, afsrc->sampc);
 	if (err)
 		warning("mixausrc: could not resample frame (%m)\n", err);
 
@@ -228,54 +239,71 @@ static void ausrc_prm_aufilt(struct ausrc_prm *ausprm,
 }
 
 
-static void ausrc_read_handler(struct auframe *afsrc, void *arg)
+static void switch_mode(struct mixstatus *st, enum mixmode mode)
 {
-	struct mixstatus *st = arg;
-	int err = 0;
-
-	if (!st->prm.srate || !st->prm.ch)
+	if (!st || st->mode == mode)
 		return;
 
-	ausrc_prm_af(&st->ausrc_prm, afsrc);
-	if (!st->ausrc_prm.srate || !st->ausrc_prm.ch)
-		return;
+	debug("mixausrc: mode %s --> %s\n",
+	      str_mixmode(st->mode), str_mixmode(mode));
+	st->mode = mode;
+}
 
-	if (!st->sampc_max || !st->nbytes_max)
-		return;
 
-	if (st->ausrc_prm.srate != st->prm.srate ||
-			st->ausrc_prm.ch != st->prm.ch)
-		err = process_resamp(st, afsrc);
-
-	if (err) {
-		st->nextmode = FM_FADEIN;
-		return;
-	}
-
-	if (!st->aubuf) {
-		err = init_aubuf(st);
-		if (err) {
-			st->nextmode = FM_FADEIN;
-			return;
-		}
-	}
-
-	if (st->sampvrs) {
-		aubuf_write(st->aubuf, st->sampvrs, st->nbytes_strm);
-	}
-	else
-		aubuf_write(st->aubuf, afsrc->sampv, st->nbytes_strm);
+static void switch_mode_locked(struct mixstatus *st, enum mixmode mode)
+{
+	mtx_lock(st->mtx_mode);
+	switch_mode(st, mode);
+	mtx_unlock(st->mtx_mode);
 }
 
 
 static void ausrc_error_handler(int err, const char *str, void *arg)
 {
 	struct mixstatus *st = arg;
-	(void) err;
-	(void) str;
+	(void)str;
 
+	/* reached EOS of ausrc */
+	debug("mixausrc: reached EOS of ausrc (%m)\n", err);
+	switch_mode_locked(st, FM_FADEIN);
 	stop_ausrc(st);
-	st->mode = FM_FADEIN;
+}
+
+
+static void ausrc_read_handler(struct auframe *afsrc, void *arg)
+{
+	struct mixstatus *st = arg;
+	int err = 0;
+
+	mtx_lock(st->mtx_ausrc);
+	if (!st->prm.srate || !st->prm.ch)
+		goto out;
+
+	ausrc_prm_af(&st->ausrc_prm, afsrc);
+	if (!st->ausrc_prm.srate || !st->ausrc_prm.ch)
+		goto out;
+
+	struct auframe afres = *afsrc;
+	if (st->ausrc_prm.srate != st->prm.srate ||
+	    st->ausrc_prm.ch != st->prm.ch) {
+		err = process_resamp(st, &afres, afsrc);
+		if (err)
+			goto out;
+	}
+
+	if (!st->aubuf) {
+		err = init_aubuf(st);
+		if (err)
+			goto out;
+	}
+
+	afres.timestamp = 0;
+	aubuf_write_auframe(st->aubuf, &afres);
+out:
+	if (err)
+		switch_mode_locked(st, FM_FADEIN);
+
+	mtx_unlock(st->mtx_ausrc);
 }
 
 
@@ -283,8 +311,8 @@ static int start_ausrc(struct mixstatus *st)
 {
 	int err;
 
+	mtx_lock(st->mtx_ausrc);
 	auresamp_init(&st->resamp);
-	st->ausrc_prm.ptime = st->ptime;
 	err = ausrc_alloc(&st->ausrc, baresip_ausrcl(), st->module,
 			  &st->ausrc_prm, st->param, ausrc_read_handler,
 			  ausrc_error_handler, st);
@@ -293,33 +321,39 @@ static int start_ausrc(struct mixstatus *st)
 		warning("mixausrc: Could not start audio source %s with "
 				"data %s.\n", st->module, st->param);
 		err = EINVAL;
-		st->mode = FM_FADEIN;
+		goto out;
 	}
 
 	/* now we are ready for next start_process */
 	st->module = mem_deref(st->module);
 	st->param = mem_deref(st->param);
-	st->sampc_max = st->ausrc_prm.srate * st->ausrc_prm.ptime *
-			st->ausrc_prm.ch / 1000;
-	if (st->sampc_strm > st->sampc_max)
-		st->sampc_max = st->sampc_strm;
 
-	st->nbytes_max = st->sampc_max * aufmt_sample_size(st->ausrc_prm.fmt);
+	struct ausrc_prm *prm = &st->ausrc_prm;
+	/* nres holds max of both streams */
+	size_t sz = aufmt_sample_size(prm->fmt);
+	size_t n = sz * prm->srate * prm->ch * PTIME / 1000;
+	if (n > st->nres)
+		st->nres = n;
 
+out:
+	if (err)
+		switch_mode_locked(st, FM_FADEIN);
+
+	mtx_unlock(st->mtx_ausrc);
 	return err;
 }
 
 
-static int stop_ausrc(struct mixstatus *st)
+static void stop_ausrc(struct mixstatus *st)
 {
 	st->ausrc   = mem_deref(st->ausrc);
+	mtx_lock(st->mtx_ausrc);
 	st->aubuf   = mem_deref(st->aubuf);
-	st->rbuf    = mem_deref(st->rbuf);
+	st->mixbuf  = mem_deref(st->mixbuf);
 	st->sampvrs = mem_deref(st->sampvrs);
-	st->sampc_max  = 0;
-	st->nbytes_max = 0;
-	st->aubuf_started = false;
-	return 0;
+	st->nres  = 0;
+	st->nmix  = 0;
+	mtx_unlock(st->mtx_ausrc);
 }
 
 
@@ -328,29 +362,38 @@ static void destruct(struct mixstatus *st)
 	stop_ausrc(st);
 	mem_deref(st->module);
 	mem_deref(st->param);
+	mem_deref(st->mtx_ausrc);
+	mem_deref(st->mtx_mode);
 }
 
 
 static void enc_destructor(void *arg)
 {
-	struct mixausrc_enc *st = (struct mixausrc_enc *) arg;
+	struct mixausrc_enc *enc = (struct mixausrc_enc *) arg;
+	mem_deref(enc->cname);
 
-	list_unlink(&st->le_priv);
-	destruct(&st->st);
+	list_unlink(&enc->le_priv);
+	destruct(&enc->st);
 }
 
 
 static void dec_destructor(void *arg)
 {
-	struct mixausrc_dec *st = (struct mixausrc_dec *)arg;
+	struct mixausrc_dec *dec = (struct mixausrc_dec *)arg;
+	mem_deref(dec->cname);
 
-	list_unlink(&st->le_priv);
-	destruct(&st->st);
+	list_unlink(&dec->le_priv);
+	destruct(&dec->st);
 }
 
 
-static void mixstatus_init(struct mixstatus *st, struct aufilt_prm *prm)
+static int mixstatus_init(struct mixstatus *st, struct aufilt_prm *prm)
 {
+	int err;
+	err  = mutex_alloc(&st->mtx_ausrc);
+	err |= mutex_alloc(&st->mtx_mode);
+	if (err)
+		return err;
 
 	stop_ausrc(st);
 
@@ -359,11 +402,13 @@ static void mixstatus_init(struct mixstatus *st, struct aufilt_prm *prm)
 	st->ausvol = 1.0f;
 	st->i_fade = 0;
 
-	/* initialize with configured values */
+	/* initialize with filter parameters */
 	st->prm = *prm;
 	st->ausrc_prm.ch = prm->ch;
 	st->ausrc_prm.fmt = prm->fmt;
 	st->ausrc_prm.srate = prm->srate;
+	st->ausrc_prm.ptime = PTIME;
+	return 0;
 }
 
 
@@ -371,31 +416,31 @@ static int encode_update(struct aufilt_enc_st **stp, void **ctx,
 			 const struct aufilt *af, struct aufilt_prm *prm,
 			 const struct audio *au)
 {
-	struct mixausrc_enc *st;
+	struct mixausrc_enc *enc;
+	int err;
 	(void)au;
 	(void)af;
-	(void)prm;
+	(void)ctx;
 
 	if (!stp || !ctx || !prm)
 		return EINVAL;
 
-	if (prm->ch!=1) {
-		warning("mixausrc: Only mono is supported.\n");
-		return EINVAL;
-	}
-
 	if (*stp)
 		return 0;
 
-	st = mem_zalloc(sizeof(*st), enc_destructor);
-	if (!st)
+	enc = mem_zalloc(sizeof(*enc), enc_destructor);
+	if (!enc)
 		return ENOMEM;
 
-	list_append(&encs, &st->le_priv, st);
-	*stp = (struct aufilt_enc_st *) st;
+	enc->st.fadetime = DEFAULT_FADE_TIME;
+	err = str_dup(&enc->cname, stream_cname(audio_strm(au)));
+	if (err)
+		return err;
 
-	mixstatus_init(&st->st, prm);
-	return 0;
+	list_append(&encs, &enc->le_priv, enc);
+	*stp = (struct aufilt_enc_st *) enc;
+
+	return mixstatus_init(&enc->st, prm);
 }
 
 
@@ -403,25 +448,31 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 			 const struct aufilt *af, struct aufilt_prm *prm,
 			 const struct audio *au)
 {
-	struct mixausrc_dec *st;
+	struct mixausrc_dec *dec;
+	int err;
 	(void)au;
 	(void)af;
+	(void)ctx;
 
-	if (!stp || !ctx)
+	if (!stp || !ctx || !prm)
 		return EINVAL;
 
 	if (*stp)
 		return 0;
 
-	st = mem_zalloc(sizeof(*st), dec_destructor);
-	if (!st)
+	dec = mem_zalloc(sizeof(*dec), dec_destructor);
+	if (!dec)
 		return ENOMEM;
 
-	list_append(&decs, &st->le_priv, st);
-	*stp = (struct aufilt_dec_st *) st;
+	dec->st.fadetime = DEFAULT_FADE_TIME;
+	err = str_dup(&dec->cname, stream_cname(audio_strm(au)));
+	if (err)
+		return err;
 
-	mixstatus_init(&st->st, prm);
-	return 0;
+	list_append(&decs, &dec->le_priv, dec);
+	*stp = (struct aufilt_dec_st *) dec;
+
+	return mixstatus_init(&dec->st, prm);
 }
 
 
@@ -510,7 +561,7 @@ static void mix_int16(struct mixstatus *st, int16_t *data, size_t n)
 	size_t i;
 	for (i = 0; i < n; i++)
 		data[i] = (int16_t) (data[i] * st->minvol +
-				(st->ausvol * st->rbuf[i]));
+				(st->ausvol * st->mixbuf[i]));
 }
 
 
@@ -519,7 +570,7 @@ static void mix_float(struct mixstatus *st, float *data, size_t n)
 	size_t i;
 	for (i = 0; i < n; i++)
 		data[i] = data[i] * st->minvol +
-			(st->ausvol * st->rbuf[i]);
+			(st->ausvol * st->mixbuf[i]);
 }
 
 
@@ -536,92 +587,95 @@ static int mixframe(struct mixstatus *st, struct auframe *af)
 }
 
 
-static void aufilt_prm_update(struct aufilt_prm *prm, struct auframe *af)
+static void aufilt_prm_update(struct mixstatus *st, struct auframe *af)
 {
-	prm->srate = af->srate;
-	prm->ch    = af->ch;
-	prm->fmt   = af->fmt;
+	if (st->prm.srate == af->srate &&
+	    st->prm.ch    == af->ch   &&
+	    st->prm.fmt   == (int) af->fmt)
+		return;
+
+	warning("mixausrc: auframe parameters do not match filter "
+		"parameters\n");
+	mtx_lock(st->mtx_ausrc);
+	st->prm.srate = af->srate;
+	st->prm.ch    = af->ch;
+	st->prm.fmt   = (int) af->fmt;
+	mtx_unlock(st->mtx_ausrc);
 }
 
 
 static int process(struct mixstatus *st, struct auframe *af)
 {
-	size_t n = af->sampc;
-	uint32_t ptime = (uint32_t) n * 1000 / (af->srate * af->ch);
 	int err = 0;
 
-	st->sampc_strm = af->sampc;
-	st->nbytes_strm = auframe_size(af);
-	if (!st->ptime) {
-		st->ptime = ptime;
-	}
-	else if (st->ptime != ptime) {
-		warning("mixausrc: ptime changed %u --> %u.\n",
-			st->ptime, ptime);
-		stop_ausrc(st);
-		st->ptime = 0;
-		st->nextmode = FM_FADEIN;
-		return EINVAL;
-	}
-
-	aufilt_prm_update(&st->prm, af);
-	if (st->mode == FM_FADEOUT && st->i_fade == st->n_fade)
-		st->mode = FM_MIX;
+	aufilt_prm_update(st, af);
 
 	/* process nextmode */
-	if (st->mode == FM_MIX && st->nextmode == FM_FADEOUT)
+	mtx_lock(st->mtx_mode);
+	if (st->mode == FM_MIX && st->nextmode == FM_FADEOUT) {
 		st->nextmode = FM_NONE;
-
-	else if (st->mode == FM_IDLE && st->nextmode == FM_FADEIN)
+	}
+	else if (st->mode == FM_IDLE && st->nextmode == FM_FADEIN) {
 		st->nextmode = FM_NONE;
-
+	}
 	else if (st->nextmode != FM_NONE) {
 		/* a command was invoked */
 		/* process nextmode */
-		if (st->mode != st->nextmode) {
-			debug("mixausrc: mode %s --> %s\n",
-					str_mixmode(st->mode),
-					str_mixmode(st->nextmode));
-			if (st->mode == FM_MIX)
-				stop_ausrc(st);
-		}
-
-		st->mode = st->nextmode;
+		switch_mode(st, st->nextmode);
 		st->nextmode = FM_NONE;
 	}
 
-	switch (st->mode) {
+	enum mixmode mode = st->mode;
+	mtx_unlock(st->mtx_mode);
+
+	switch (mode) {
 	case FM_FADEIN: {
-		err = fadeframe(st, af, st->mode);
+		err = fadeframe(st, af, mode);
 		if (st->i_fade >= st->n_fade) {
 			st->i_fade = 0;
-			st->mode = FM_IDLE;
+			switch_mode_locked(st, FM_IDLE);
 		}
 	}
 	break;
 	case FM_FADEOUT: {
-		err = fadeframe(st, af, st->mode);
+		err = fadeframe(st, af, mode);
 		if (st->i_fade >= st->n_fade) {
 			st->i_fade = 0;
-			st->mode = FM_MIX;
+			switch_mode_locked(st, FM_MIX);
 		}
 	}
 	break;
 	case FM_MIX: {
+		size_t n = auframe_size(af);
+		size_t sz = aufmt_sample_size(af->fmt);
+		if (!st->nres) {
+			st->nres = sz * au_calc_nsamp(af->srate,
+						      af->ch, PTIME);
+			st->nmix = n;
+		}
+
+		if (!st->nres || !st->nmix) {
+			warning("mixausrc: nres or nmix is zero\n");
+			return EINVAL;
+		}
+
 		if (!st->ausrc) {
 			start_ausrc(st);
 			clear_frame(st, af);
 		}
-		else if (aubuf_cur_size(st->aubuf) >= st->nbytes_strm) {
-			st->aubuf_started = true;
-			aubuf_read(st->aubuf,
-				   (void *) st->rbuf, st->nbytes_strm);
+		else {
+			if (!st->mixbuf || n > st->nmix) {
+				st->nmix = n;
+				init_mixbuf(st);
+			}
+
+			struct auframe afmix;
+			auframe_init(&afmix, af->fmt, st->mixbuf, af->sampc,
+				     af->srate, af->ch);
+			aubuf_read_auframe(st->aubuf, &afmix);
 			/* now mix */
 			err = mixframe(st, af);
 		}
-		else {
-			clear_frame(st, af);
-		}
 	}
 	break;
 	default:
@@ -634,17 +688,17 @@ static int process(struct mixstatus *st, struct auframe *af)
 
 static int encode(struct aufilt_enc_st *aufilt_enc_st, struct auframe *af)
 {
-	struct mixausrc_enc *mixausrc = (void *)aufilt_enc_st;
+	struct mixausrc_enc *enc= (struct mixausrc_enc *)aufilt_enc_st;
 
-	return process(&mixausrc->st, af);
+	return process(&enc->st, af);
 }
 
 
 static int decode(struct aufilt_dec_st *aufilt_dec_st, struct auframe *af)
 {
-	struct mixausrc_dec *mixausrc = (void *)aufilt_dec_st;
+	struct mixausrc_dec *dec = (struct mixausrc_dec *)aufilt_dec_st;
 
-	return process(&mixausrc->st, af);
+	return process(&dec->st, af);
 }
 
 
@@ -664,33 +718,41 @@ static float conv_volume(const struct pl *pl)
 static void print_usage(const char *name)
 {
 	info("mixausrc: Missing parameters. Usage:\n"
-			"%s <module> <param> [minvol] [ausvol]\n"
-			"module  The audio source module.\n"
-			"param   The audio source parameter. If this is an"
-			" audio file,\n"
-			"        then you have to specify the full path.\n"
-			"minvol  The minimum fade out mic volume (0-100).\n"
-			"ausvol  The audio source volume (0-100).\n", name);
+		"%s <module> <param> [minvol] [ausvol] [cname=string] "
+				     "[fadetime=ms]\n"
+		"module   The audio source module\n"
+		"param    The audio source parameter. If this is an audio "
+			  "file,\n"
+		"         then you have to specify the full path\n"
+		"minvol   The minimum fade out mic volume (0-100)\n"
+		"ausvol   The audio source volume (0-100)\n"
+		"cname    Canonical end-point identifier\n"
+		"fadetime Sets the fade-out/-in time in [ms]", name);
 }
 
 
 static int start_process(struct mixstatus* st, const char *name,
-		const struct cmd_arg *carg)
+			 const struct cmd_arg *carg)
 {
 	int err = 0;
 	struct pl pl1 = PL_INIT;
 	struct pl pl2 = PL_INIT;
 	struct pl pl3 = PL_INIT;
 	struct pl pl4 = PL_INIT;
+	struct pl pl5 = PL_INIT;
 
 	if (!carg || !str_isset(carg->prm)) {
 		print_usage(name);
 		return EINVAL;
 	}
 
-	if (st->mode != FM_IDLE && st->mode != FM_FADEIN) {
-		warning("mixausrc: %s is not possible while mode is %s\n",
-				name, str_mixmode(st->mode));
+	mtx_lock(st->mtx_mode);
+	enum mixmode mode = st->mode;
+	enum mixmode nextmode = st->nextmode;
+	mtx_unlock(st->mtx_mode);
+	if (mode != FM_IDLE || nextmode != FM_NONE) {
+		warning("mixausrc: %s is not possible while mode is %s/%s\n",
+			name, str_mixmode(mode), str_mixmode(nextmode));
 		return EINVAL;
 	}
 
@@ -717,12 +779,21 @@ static int start_process(struct mixstatus* st, const char *name,
 	st->minvol = pl_isset(&pl3) ? conv_volume(&pl3) : 0.0f;
 	st->ausvol = pl_isset(&pl4) ? conv_volume(&pl4) : 1.0f;
 	st->i_fade = 0;
-	st->n_fade = (DEFAULT_FADE_TIME * st->ausrc_prm.srate) / 1000;
+	cparam_decode(carg->prm, "fadetime", &pl5);
+	if (pl_isset(&pl5)) {
+		uint32_t v = pl_u32(&pl5);
+		if (v > 0)
+			st->fadetime = v;
+	}
+
+	st->n_fade = (st->fadetime * st->prm.srate) / 1000;
 	st->delta_fade = (1.0f - st->minvol) / st->n_fade;
 
 	stop_ausrc(st);
 	ausrc_prm_aufilt(&st->ausrc_prm, &st->prm);
+	mtx_lock(st->mtx_mode);
 	st->nextmode = FM_FADEOUT;
+	mtx_unlock(st->mtx_mode);
 
 	return 0;
 }
@@ -753,12 +824,68 @@ static const struct cmd cmdv[] = {
 	" stream.", enc_mix_start},
 {"mixausrc_dec_start", 0, CMD_PRM, "Start mixing audio source into decoding"
 	" stream.", dec_mix_start},
-{"mixausrc_enc_stop",  0, 0, "Stop mixing of encoding stream.", enc_mix_stop},
-{"mixausrc_dec_stop",  0, 0, "Stop mixing of decoding stream.", dec_mix_stop},
+{"mixausrc_enc_stop",  0, CMD_PRM, "Stop mixing of encoding stream.",
+	enc_mix_stop},
+{"mixausrc_dec_stop",  0, CMD_PRM, "Stop mixing of decoding stream.",
+	dec_mix_stop},
 
 };
 
 
+static struct mixausrc_enc *enc_find(const struct cmd_arg *carg)
+{
+	struct pl cname = PL_INIT;
+	cparam_decode(carg->prm, "cname", &cname);
+
+	if (!pl_isset(&cname))
+		return encs.head ? encs.head->data : NULL;
+
+	char *cname_str;
+	int err = pl_strdup(&cname_str, &cname);
+	if (err)
+		return NULL;
+
+	struct mixausrc_enc *enc = NULL;
+	for (struct le *le = encs.head; le; le = le->next) {
+		enc = le->data;
+		if (!re_regex(enc->cname, str_len(enc->cname), cname_str))
+			goto out;
+	}
+
+	warning("mixausrc: no encoding stream with cname=%r\n",	&cname);
+out:
+	mem_deref(cname_str);
+	return enc;
+}
+
+
+static struct mixausrc_dec *dec_find(const struct cmd_arg *carg)
+{
+	struct pl cname = PL_INIT;
+	cparam_decode(carg->prm, "cname", &cname);
+
+	if (!pl_isset(&cname))
+		return decs.head ? decs.head->data : NULL;
+
+	char *cname_str;
+	int err = pl_strdup(&cname_str, &cname);
+	if (err)
+		return NULL;
+
+	struct mixausrc_dec *dec = NULL;
+	for (struct le *le = decs.head; le; le = le->next) {
+		dec = le->data;
+		if (!re_regex(dec->cname, str_len(dec->cname), cname_str))
+			goto out;
+	}
+
+	warning("mixausrc: no decoding stream with cname=%r\n",	&cname);
+out:
+	mem_deref(cname_str);
+	return dec;
+}
+
+
 /**
  * Start mixing audio source into encoding stream.
  *
@@ -770,7 +897,7 @@ static const struct cmd cmdv[] = {
 static int enc_mix_start(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
-	struct mixausrc_enc *st;
+	struct mixausrc_enc *enc;
 	(void)pf;
 
 	if (!list_count(&encs)) {
@@ -778,10 +905,12 @@ static int enc_mix_start(struct re_printf *pf, void *arg)
 		return EINVAL;
 	}
 
-	st = encs.head->data;
+	enc = enc_find(carg);
+	if (!enc)
+		return EINVAL;
 
 	debug("mixausrc: %s\n", __func__);
-	return start_process(&st->st, cmdv[0].name, carg);
+	return start_process(&enc->st, cmdv[0].name, carg);
 }
 
 
@@ -804,42 +933,39 @@ static int dec_mix_start(struct re_printf *pf, void *arg)
 		return EINVAL;
 	}
 
-	dec = decs.head->data;
+	dec = dec_find(carg);
+	if (!dec)
+		return EINVAL;
 
 	debug("mixausrc: %s\n", __func__);
 	return start_process(&dec->st, cmdv[1].name, carg);
 }
 
 
-static int stop_process(struct mixstatus *st)
-{
-
-	st->nextmode = FM_FADEIN;
-	return 0;
-}
-
-
 /**
  * Stop mixing of encoding stream.
  *
  * @param pf		Print handler for debug output
- * @param unused	Not used.
+ * @param arg		Command argument. See \ref cmdv !
  *
  * @return	0 if success, otherwise error code.
  */
-static int enc_mix_stop(struct re_printf *pf, void *unused)
+static int enc_mix_stop(struct re_printf *pf, void *arg)
 {
+	const struct cmd_arg *carg = arg;
 	struct mixausrc_enc *enc;
 	(void)pf;
-	(void)unused;
 
 	if (!list_count(&encs))
 		return EINVAL;
 
-	enc = encs.head->data;
+	enc = enc_find(carg);
+	if (!enc)
+		return EINVAL;
 
 	debug("mixausrc: %s\n", __func__);
-	return stop_process(&enc->st);
+	switch_mode_locked(&enc->st, FM_FADEIN);
+	return 0;
 }
 
 
@@ -847,23 +973,26 @@ static int enc_mix_stop(struct re_printf *pf, void *unused)
  * Stop mixing of decoding stream.
  *
  * @param pf		Print handler for debug output
- * @param unused	Not used.
+ * @param arg		Command argument. See \ref cmdv !
  *
  * @return	0 if success, otherwise error code.
  */
-static int dec_mix_stop(struct re_printf *pf, void *unused)
+static int dec_mix_stop(struct re_printf *pf, void *arg)
 {
+	const struct cmd_arg *carg = arg;
 	struct mixausrc_dec *dec;
 	(void)pf;
-	(void)unused;
 
 	if (!list_count(&decs))
 		return EINVAL;
 
-	dec = decs.head->data;
+	dec = dec_find(carg);
+	if (!dec)
+		return EINVAL;
 
 	debug("mixausrc: %s\n", __func__);
-	return stop_process(&dec->st);
+	switch_mode_locked(&dec->st, FM_FADEIN);
+	return 0;
 }
 
 
