@@ -83,7 +83,7 @@ struct call {
 	enum sdp_dir estvdir;      /**< Established video direction         */
 	bool use_video;
 	bool use_rtp;
-	char *user_data;           /**< User data related to the call       */
+	struct pl *user_data;      /**< User data related to the call       */
 };
 
 
@@ -733,7 +733,6 @@ int call_streams_alloc(struct call *call)
 	struct account *acc = call->acc;
 	struct stream_param strm_prm;
 	struct le *le;
-	int label = 0;
 	int err;
 
 	memset(&strm_prm, 0, sizeof(strm_prm));
@@ -771,9 +770,6 @@ int call_streams_alloc(struct call *call)
 	FOREACH_STREAM {
 		struct stream *strm = le->data;
 
-		sdp_media_set_lattr(stream_sdpmedia(strm), true,
-				    "label", "%d", ++label);
-
 		stream_set_session_handlers(strm, stream_mnatconn_handler,
 					    stream_rtpestab_handler,
 					    stream_rtcp_handler,
@@ -3166,6 +3162,29 @@ struct call *call_find_linenum(const struct list *calls, uint32_t linenum)
 }
 
 
+/**
+ * Find a call by call-id
+ *
+ * @param calls   List of calls
+ * @param id      Call-id pointer-length string
+ *
+ * @return Call object if found, NULL if not found
+ */
+struct call *call_find_id_pl(const struct list *calls, const struct pl *id)
+{
+	struct le *le;
+
+	for (le = list_head(calls); le; le = le->next) {
+		struct call *call = le->data;
+
+		if (0 == pl_strcmp(id, call->id))
+			return call;
+	}
+
+	return NULL;
+}
+
+
 /**
  * Find a call by call-id
  *
@@ -3335,6 +3354,7 @@ bool call_supported(struct call *call, uint16_t tags)
 	return (call->supported & tags) == tags;
 }
 
+
 /**
  * Get the user data for the call
  *
@@ -3342,11 +3362,12 @@ bool call_supported(struct call *call, uint16_t tags)
  *
  * @return Call's user data
  */
-const char *call_user_data(const struct call *call)
+const struct pl *call_user_data(const struct call *call)
 {
 	return call ? call->user_data : NULL;
 }
 
+
 /**
  * Set the user data of the call
  *
@@ -3354,20 +3375,17 @@ const char *call_user_data(const struct call *call)
  * @param user_data User data to be set
  * @return int
  */
-
-int call_set_user_data(struct call *call, const char *user_data)
+int call_set_user_data(struct call *call, const struct pl *user_data)
 {
 	if (!call)
 		return EINVAL;
 
 	call->user_data = mem_deref(call->user_data);
+	if (!pl_isset(user_data))
+		return 0;
 
-	int err = str_dup(&call->user_data, user_data);
-
-	if (err)
-		return err;
-
-	return 0;
+	call->user_data = pl_alloc_dup(user_data);
+	return call->user_data ? 0 : ENOMEM;
 }
 
 
