@@ -198,6 +198,7 @@ static void rtprecv_periodic(void *arg)
 		}
 	}
 	else {
+		tmr_cancel(&rx->tmr_decode);
 		udp_thread_detach(rtp_sock(rx->rtp));
 		udp_thread_detach(rtcp_sock(rx->rtp));
 		re_cancel();
@@ -208,9 +209,11 @@ static void rtprecv_periodic(void *arg)
 static int rtprecv_thread(void *arg)
 {
 	struct rtp_receiver *rx = arg;
-	int err;
 
-	re_thread_init();
+	int err = re_thread_init();
+	if (err)
+		return err;
+
 	info("rtp_receiver: RTP RX thread started\n");
 	tmr_start(&rx->tmr, 10, rtprecv_periodic, rx);
 
@@ -733,6 +736,7 @@ static void destructor(void *arg)
 	}
 
 	tmr_cancel(&rx->tmr_decode);
+
 	mem_deref(rx->metric);
 	mem_deref(rx->name);
 	mem_deref(rx->mtx);
