@@ -281,7 +281,7 @@ int ua_register(struct ua *ua)
  */
 int ua_fallback(struct ua *ua)
 {
-	if (!ua || !ua_account(ua)->fbregint)
+	if (!ua || !account_fbregint(ua_account(ua)))
 		return 0;
 
 	debug("ua: ua_fallback %s\n", account_aor(ua->acc));
@@ -2325,8 +2325,9 @@ bool ua_req_allowed(const struct ua *ua, const struct sip_msg *msg)
 
 
 /**
- * Return false if filter_registrar is active for the message transport and the
- * SIP request IP does not match the server IP
+ * Return false if filter_registrar is active for the message transport,
+ * UA's account has ;check_origin=yes, and the SIP request IP does not
+ * match the server IP
  *
  * @param ua  User-Agent
  * @param msg SIP message
@@ -2343,6 +2344,9 @@ bool ua_req_check_origin(const struct ua *ua, const struct sip_msg *msg)
 	if (!u32mask_enabled(uag_cfg()->reg_filt, msg->tp))
 		return true;
 
+	if (!ua->acc->check_origin)
+		return true;
+
 	for (le = ua->regl.head; le; le = le->next) {
 		struct reg *reg = le->data;
 		if (sa_cmp(reg_paddr(reg), &msg->src, SA_ADDR))
