@@ -113,6 +113,34 @@ int uag_hold_others(struct call *call)
 }
 
 
+/**
+ * Find call with given id
+ *
+ * @param id  Call-id pointer-length string
+ *
+ * @return The call if found, otherwise NULL.
+ */
+struct call *uag_call_find_pl(const struct pl *id)
+{
+	struct le *le = NULL;
+	struct ua *ua = NULL;
+	struct call *call = NULL;
+
+	if (!pl_isset(id))
+		return NULL;
+
+	for (le = list_head(&uag.ual); le; le = le->next) {
+		ua = le->data;
+
+		call = call_find_id_pl(ua_calls(ua), id);
+		if (call)
+			break;
+	}
+
+	return call;
+}
+
+
 /**
  * Find call with given id
  *
@@ -157,10 +185,11 @@ void uag_filter_calls(call_list_h *listh, call_match_h *matchh, void *arg)
 
 	for (leu = list_head(uag_list()); leu; leu = leu->next) {
 		struct ua *ua = leu->data;
-		struct le *lec;
+		struct le *lec = list_tail(ua_calls(ua));
 
-		for (lec = list_tail(ua_calls(ua)); lec; lec = lec->prev) {
+		while (lec) {
 			struct call *call = lec->data;
+			lec = lec->prev;
 
 			if (!matchh || matchh(call, arg))
 				listh(call, arg);
@@ -862,7 +891,7 @@ struct ua *uag_find(const struct pl *cuser)
 	for (le = uag.ual.head; le; le = le->next) {
 		struct ua *ua = le->data;
 
-		if (0 == pl_strcasecmp(cuser, ua_local_cuser(ua)))
+		if (0 == pl_strcasecmp(cuser, ua_cuser(ua)))
 			return ua;
 	}
 
